@@ -1,66 +1,47 @@
 from __future__ import unicode_literals, division, absolute_import
+import atexit
+from contextlib import contextmanager
+import signal
 import os
 import sys
 import shutil
 import logging
+import threading
+import pkg_resources
+import traceback
 import yaml
-import codecs
-import atexit
 from datetime import datetime, timedelta
 
 import sqlalchemy
 from sqlalchemy.orm import sessionmaker
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.pool import SingletonThreadPool
+from sqlalchemy.exc import OperationalError
 
+# These need to be declared before we start importing from other flexget modules, since they might import them
+Base = declarative_base()
+_Session = sessionmaker()
+class Session(_Session):
+    def commit(self, *args, **kwargs):
+        if not manager.has_lock:
+            if manager.unit_test:
+                raise Exception('Cannot commit to database without lock.')
+            log.error('BUG: Database writes should not be tried when there is no database lock. Please Report.')
+            traceback.print_stack()
+        else:
+            super(Session, self).commit(*args, **kwargs)
+
+from flexget import config_schema, db_schema
 from flexget.event import fire_event
-from flexget import config_schema
+from flexget.ipc import IPCServer, IPCClient
+from flexget.scheduler import Scheduler
+from flexget.utils.tools import pid_exists
 
 log = logging.getLogger('manager')
 
-Base = declarative_base()
-Session = sessionmaker()
 manager = None
 DB_CLEANUP_INTERVAL = timedelta(days=7)
 
-# Validator that handles root structure of config.
-_root_config_schema = {'type': 'object', 'additionalProperties': False}
-# TODO: Is /schema/root this the best place for this?
-config_schema.register_schema('/schema/root', _root_config_schema)
-
-
-def register_config_key(key, schema, required=False):
-    """ Registers a valid root level key for the config.
-
-    :param string key:
-      Name of the root level key being registered.
-    :param dict schema:
-      Schema for the key.
-    :param bool required:
-      Specify whether this is a mandatory key.
-    """
-    _root_config_schema.setdefault('properties', {})[key] = schema
-    if required:
-        _root_config_schema.setdefault('required', []).append(key)
-
-
-def useExecLogging(func):
-    """
-    Decorator for setting task name to log messages.
-    """
-
-    def wrapper(self, *args, **kw):
-        # Set the task name in the logger
-        from flexget import logger
-        import time
-        logger.set_execution(str(time.time()))
-        try:
-            return func(self, *args, **kw)
-        finally:
-            logger.set_execution('')
-
-    return wrapper
-
 
 class Manager(object):
 
@@ -70,24 +51,34 @@ class Manager(object):
 
     * manager.startup
 
-      After manager has been initialized. This is when application becomes ready to use
+      After manager has been initialized. This is when application becomes ready to use, however no database lock is
+      present, so the database must not be modified on this event.
+
+    * manager.lock-acquired
+
+      The manager does not always require a lock on startup, if one is requested, this event will run when it has been
+      acquired successfully
+
+    * manager.config-loaded
+
+      When a valid configuration has been (re)loaded, this event is fired
 
     * manager.upgrade
 
-      Upgrade plugin database schemas etc
-
-    * manager.execute.started
-
-      When execute is about the be started, this happens before any task phases occur
-      including on_process_start
-
-    * manager.execute.completed
-
-      After manager has executed all Tasks
+      If any plugins have declared a newer schema version than exists in the database, this event will be fired to
+      allow plugins to upgrade their tables
 
     * manager.shutdown
 
       When the manager is exiting
+
+    * manager.execute.completed
+
+      If execution in current process was completed
+
+    * manager.daemon.started
+    * manager.daemon.completed
+    * manager.db_cleanup
     """
 
     unit_test = False
@@ -99,27 +90,24 @@ class Manager(object):
         """
         global manager
         assert not manager, 'Only one instance of Manager should be created at a time!'
-        manager = self
         self.options = options
         self.config_base = None
         self.config_name = None
+        self.config_path = None
         self.db_filename = None
         self.engine = None
         self.lockfile = None
         self.database_uri = None
         self.db_upgraded = False
+        self._has_lock = False
 
         self.config = {}
-        self.tasks = {}
 
+        self.scheduler = Scheduler(self)
+        self.ipc_server = IPCServer(self, options.ipc_port)
+        manager = self
         self.initialize()
 
-        # check if stdout is not connected to terminal (cron run)
-        #if not os.isatty(sys.stdout.fileno()):
-        #    log.warning('It appears you\'re running from CRON')
-        #else:
-        #    log.warning('It appears you\'re running from terminal')
-
         # cannot be imported at module level because of circular references
         from flexget.utils.simple_persistence import SimplePersistence
         self.persist = SimplePersistence('manager')
@@ -128,11 +116,15 @@ class Manager(object):
         log.debug('sys.getfilesystemencoding: %s' % sys.getfilesystemencoding())
         log.debug('os.path.supports_unicode_filenames: %s' % os.path.supports_unicode_filenames)
 
-        fire_event('manager.upgrade', self)
-        if manager.db_upgraded:
-            fire_event('manager.db_upgraded', self)
+        if db_schema.upgrade_required():
+            log.info('Database upgrade is required. Attempting now.')
+            # Make sure not to fire the lock-acquired event yet
+            # TODO: Detect if any database upgrading is needed and acquire the lock only in one place
+            with self.acquire_lock(event=False):
+                fire_event('manager.upgrade', self)
+                if manager.db_upgraded:
+                    fire_event('manager.db_upgraded', self)
         fire_event('manager.startup', self)
-        self.db_cleanup()
 
     def __del__(self):
         global manager
@@ -141,19 +133,140 @@ class Manager(object):
     def initialize(self):
         """Separated from __init__ so that unit tests can modify options before loading config."""
         self.setup_yaml()
-        self.find_config()
-        self.acquire_lock()
+        self.find_config(create=(self.options.cli_command == 'webui'))
         self.init_sqlalchemy()
+        fire_event('manager.before_config_load', self)
+        self.load_config()
+        fire_event('manager.before_config_validate', self)
         errors = self.validate_config()
         if errors:
             for error in errors:
                 log.critical("[%s] %s", error.json_pointer, error.message)
+            self.shutdown(finish_queue=False)
+            sys.exit(1)
+        fire_event('manager.config-loaded', self)
+
+    @property
+    def tasks(self):
+        """A list of tasks in the config"""
+        if not self.config:
+            return []
+        return self.config.get('tasks', {}).keys()
+
+    @property
+    def has_lock(self):
+        return self._has_lock
+
+    def run_cli_command(self):
+        """
+        Starting point when executing from commandline, dispatch execution
+        to correct destination.
+
+        * :meth:`.execute_command`
+        * :meth:`.daemon_command`
+        * :meth:`.webui_command`
+        * CLI plugin callback function
+        """
+        command = self.options.cli_command
+        options = getattr(self.options, command)
+        # First check for built-in commands
+        if command == 'execute':
+            self.execute_command(options)
+        elif command == 'daemon':
+            self.daemon_command(options)
+        elif command == 'webui':
+            self.webui_command(options)
+        else:
+            # Otherwise dispatch the command to the callback function
+            options.cli_command_callback(self, options)
+
+    def execute_command(self, options):
+        """
+        Send execute command to daemon through IPC or perform execution
+        on current process.
+
+        Fires events:
+
+        * manager.execute.completed
+
+        :param options: argparse options
+        """
+        # If a daemon is started, send the execution to the daemon
+        ipc_info = self.check_ipc_info()
+        if ipc_info:
+            client = IPCClient(ipc_info['port'], ipc_info['password'])
+            client.execute(dict(options))
+            self.shutdown()
             return
-        self.create_tasks()
+        # Otherwise we run the execution ourselves
+        with self.acquire_lock():
+            fire_event('manager.execute.started', self)
+            self.scheduler.start(run_schedules=False)
+            self.scheduler.execute(options)
+            self.scheduler.shutdown(finish_queue=True)
+            try:
+                self.scheduler.wait()
+            except KeyboardInterrupt:
+                log.error('Got ctrl-c exiting after this task completes. Press ctrl-c again to abort this task.')
+            else:
+                fire_event('manager.execute.completed', self)
+            self.shutdown(finish_queue=False)
+
+    def daemon_command(self, options):
+        """
+        Fires events:
+
+        * manager.daemon.started
+        * manager.daemon.completed
+
+        :param options: argparse options
+        """
+        if options.action == 'start':
+            if options.daemonize:
+                self.daemonize()
+            with self.acquire_lock():
+                signal.signal(signal.SIGTERM, self._handle_sigterm)
+                self.ipc_server.start()
+                fire_event('manager.daemon.started', self)
+                self.scheduler.start()
+                try:
+                    self.scheduler.wait()
+                except KeyboardInterrupt:
+                    log.info('Got ctrl-c, shutting down.')
+                    fire_event('manager.daemon.completed', self)
+                    self.shutdown(finish_queue=False)
+        elif options.action == 'stop':
+            ipc_info = self.check_ipc_info()
+            if ipc_info:
+                client = IPCClient(ipc_info['port'], ipc_info['password'])
+                client.shutdown()
+                self.shutdown()
+            else:
+                log.error('There does not appear to be a daemon running.')
+
+    def webui_command(self, options):
+        """
+        :param options: argparse options
+        """
+        try:
+            pkg_resources.require('flexget[webui]')
+        except pkg_resources.DistributionNotFound as e:
+            log.error('Dependency not met. %s' % e)
+            log.error('Webui dependencies not installed. You can use `pip install flexget[webui]` to install them.')
+            self.shutdown()
+            return
+        if options.daemonize:
+            self.daemonize()
+        from flexget.ui import webui
+        with self.acquire_lock():
+            webui.start(self)
+
+    def _handle_sigterm(self, signum, frame):
+        log.info('Got SIGTERM. Shutting down.')
+        self.shutdown(finish_queue=False)
 
     def setup_yaml(self):
-        """ Set up the yaml loader to return unicode objects for strings by default
-        """
+        """Sets up the yaml loader to return unicode objects for strings by default"""
 
         def construct_yaml_str(self, node):
             # Override the default string handling function
@@ -178,32 +291,25 @@ class Manager(object):
         yaml.Dumper.increase_indent = increase_indent_wrapper(yaml.Dumper.increase_indent)
         yaml.SafeDumper.increase_indent = increase_indent_wrapper(yaml.SafeDumper.increase_indent)
 
-    def find_config(self):
-        """Find the configuration file and then call :meth:`.load_config` to load it"""
+    def find_config(self, create=False):
+        """
+        Find the configuration file.
+
+        :param bool create: If a config file is not found, and create is True, one will be created in the home folder
+        """
         startup_path = os.path.dirname(os.path.abspath(sys.path[0]))
         home_path = os.path.join(os.path.expanduser('~'), '.flexget')
         current_path = os.getcwd()
         exec_path = sys.path[0]
 
         config_path = os.path.dirname(self.options.config)
-        path_given = config_path != ''
 
         possible = []
-        if path_given:
+        if config_path != '':
             # explicit path given, don't try anything too fancy
             possible.append(self.options.config)
         else:
             log.debug('Figuring out config load paths')
-            # normal lookup locations
-            possible.append(startup_path)
-            possible.append(home_path)
-            if sys.platform.startswith('win'):
-                # On windows look in ~/flexget as well, as explorer does not let you create a folder starting with a dot
-                possible.append(os.path.join(os.path.expanduser('~'), 'flexget'))
-            else:
-                # The freedesktop.org standard config location
-                xdg_config = os.environ.get('XDG_CONFIG_HOME', os.path.join(os.path.expanduser('~'), '.config'))
-                possible.append(os.path.join(xdg_config, 'flexget'))
             # for virtualenv / dev sandbox
             from flexget import __version__ as version
             if version == '{git}':
@@ -211,35 +317,53 @@ class Manager(object):
                 possible.append(os.path.join(exec_path, '..'))
                 possible.append(current_path)
                 possible.append(exec_path)
+            # normal lookup locations
+            possible.append(startup_path)
+            possible.append(home_path)
+            if sys.platform.startswith('win'):
+                # On windows look in ~/flexget as well, as explorer does not let you create a folder starting with a dot
+                home_path = os.path.join(os.path.expanduser('~'), 'flexget')
+                possible.append(home_path)
+            else:
+                # The freedesktop.org standard config location
+                xdg_config = os.environ.get('XDG_CONFIG_HOME', os.path.join(os.path.expanduser('~'), '.config'))
+                possible.append(os.path.join(xdg_config, 'flexget'))
 
         for path in possible:
             config = os.path.join(path, self.options.config)
             if os.path.exists(config):
                 log.debug('Found config: %s' % config)
-                self.load_config(config)
-                return
-        log.info('Tried to read from: %s' % ', '.join(possible))
-        raise IOError('Failed to find configuration file %s' % self.options.config)
+                break
+        else:
+            if not create:
+                log.info('Tried to read from: %s' % ', '.join(possible))
+                log.critical('Failed to find configuration file %s' % self.options.config)
+                sys.exit(1)
+            config = os.path.join(home_path, self.options.config)
+            log.info('Config file %s not found. Creating new config %s' % (self.options.config, config))
+            with open(config, 'w') as newconfig:
+                # Write empty tasks to the config
+                newconfig.write(yaml.dump({'tasks': {}}))
 
-    def load_config(self, config_path):
+        self.config_path = config
+        self.config_name = os.path.splitext(os.path.basename(config))[0]
+        self.config_base = os.path.normpath(os.path.dirname(config))
+        self.lockfile = os.path.join(self.config_base, '.%s-lock' % self.config_name)
+
+    def load_config(self):
         """
         .. warning::
 
            Calls sys.exit(1) if configuration file could not be loaded.
            This is something we probably want to change.
-
-        :param string config_path: Path to configuration file
         """
-        with open(config_path, 'rb') as file:
-            config = file.read()
+        with open(self.config_path, 'rb') as f:
+            config = f.read()
         try:
             config = config.decode('utf-8')
         except UnicodeDecodeError:
             log.critical('Config file must be UTF-8 encoded.')
             sys.exit(1)
-        if not self.options.quiet:
-            # pre-check only when running without --cron
-            self.pre_check_config(config)
         try:
             self.config = yaml.safe_load(config) or {}
         except Exception as e:
@@ -285,9 +409,6 @@ class Manager(object):
             sys.exit(1)
 
         # config loaded successfully
-        self.config_name = os.path.splitext(os.path.basename(config_path))[0]
-        self.config_base = os.path.normpath(os.path.dirname(config_path))
-        self.lockfile = os.path.join(self.config_base, '.%s-lock' % self.config_name)
         log.debug('config_name: %s' % self.config_name)
         log.debug('config_base: %s' % self.config_base)
 
@@ -302,8 +423,9 @@ class Manager(object):
     def config_changed(self):
         """Makes sure that all tasks will have the config_modified flag come out true on the next run.
         Useful when changing the db and all tasks need to be completely reprocessed."""
-        for task in self.tasks.values():
-            task.config_changed()
+        from flexget.task import config_changed
+        for task in self.tasks:
+            config_changed(task)
 
     def pre_check_config(self, config):
         """Checks configuration file for common mistakes that are easily detectable"""
@@ -419,9 +541,8 @@ class Manager(object):
         Check all root level keywords are valid.
 
         :returns: A list of `ValidationError`s
-
         """
-        return config_schema.process_config(self.config, _root_config_schema)
+        return config_schema.process_config(self.config)
 
     def init_sqlalchemy(self):
         """Initialize SQLAlchemy"""
@@ -466,11 +587,14 @@ class Manager(object):
             sys.exit(1)
         Session.configure(bind=self.engine)
         # create all tables, doesn't do anything to existing tables
-        from sqlalchemy.exc import OperationalError
         try:
-            if self.options.reset or self.options.del_db:
-                log.verbose('Deleting everything from database ...')
-                Base.metadata.drop_all(bind=self.engine)
+            def before_table_create(event, target, bind, tables=None, **kw):
+                if tables:
+                    # We need to acquire a lock if we are creating new tables
+                    # TODO: Detect if any database upgrading is needed and acquire the lock only in one place
+                    self.acquire_lock(event=False).__enter__()
+
+            Base.metadata.append_ddl_listener('before-create', before_table_create)
             Base.metadata.create_all(bind=self.engine)
         except OperationalError as e:
             if os.path.exists(self.db_filename):
@@ -481,200 +605,178 @@ class Manager(object):
                                      (e.message, self.config_base)
             raise Exception(e.message)
 
+    def _read_lock(self):
+        """
+        Read the values from the lock file. Returns None if there is no current lock file.
+        """
+        if self.lockfile and os.path.exists(self.lockfile):
+            result = {}
+            with open(self.lockfile) as f:
+                lines = filter(None, f.readlines())
+            for line in lines:
+                key, value = line.split(b':', 1)
+                result[key.strip().lower()] = value.strip()
+            for key in result:
+                if result[key].isdigit():
+                    result[key] = int(result[key])
+            if not pid_exists(result['pid']):
+                return None
+            return result
+        return None
+
     def check_lock(self):
-        """Checks if there is already a lock, returns True if there is."""
-        if os.path.exists(self.lockfile):
-            # check the lock age
-            lock_time = datetime.fromtimestamp(os.path.getmtime(self.lockfile))
-            if (datetime.now() - lock_time).seconds > 36000:
-                log.warning('Lock file over 10 hour in age, ignoring it ...')
-            else:
-                return True
-        return False
+        """Returns True if there is a lock on the database."""
+        lock_info = self._read_lock()
+        if not lock_info:
+            return False
+        # Don't count it if we hold the lock
+        if os.getpid() == lock_info['pid']:
+            return False
+        return True
 
-    def acquire_lock(self):
-        if self.options.log_start:
-            log.info('FlexGet started (PID: %s)' % os.getpid())
+    def check_ipc_info(self):
+        """If a daemon has a lock on the database, return info to connect to IPC."""
+        lock_info = self._read_lock()
+        if lock_info and 'port' in lock_info:
+            return lock_info
+        return None
 
-        # Exit if there is an existing lock.
-        if self.check_lock():
-            if not self.options.quiet:
-                f = file(self.lockfile)
-                pid = f.read()
-                f.close()
-                print >> sys.stderr, 'Another process (%s) is running, will exit.' % pid.strip()
-                print >> sys.stderr, 'If you\'re sure there is no other instance running, delete %s' % self.lockfile
-            sys.exit(1)
+    @contextmanager
+    def acquire_lock(self, event=True):
+        """
+        :param bool event: If True, the 'manager.lock-acquired' event will be fired after a lock is obtained
+        """
+        acquired = False
+        try:
+            # Don't do anything if we already have a lock. This means only the outermost call will release the lock file
+            if not self._has_lock:
+                # Exit if there is an existing lock.
+                if self.check_lock():
+                    with open(self.lockfile) as f:
+                        pid = f.read()
+                    print >> sys.stderr, 'Another process (%s) is running, will exit.' % pid.split('\n')[0]
+                    print >> sys.stderr, 'If you\'re sure there is no other instance running, delete %s' % self.lockfile
+                    sys.exit(1)
 
-        f = file(self.lockfile, 'w')
-        f.write('PID: %s\n' % os.getpid())
-        f.close()
-        atexit.register(self.release_lock)
+                self._has_lock = True
+                self.write_lock()
+                acquired = True
+                if event:
+                    fire_event('manager.lock-acquired', self)
+            yield
+        finally:
+            if acquired:
+                self.release_lock()
+                self._has_lock = False
+
+    def write_lock(self, ipc_info=None):
+        assert self._has_lock
+        with open(self.lockfile, 'w') as f:
+            f.write(b'PID: %s\n' % os.getpid())
+            if ipc_info:
+                for key in sorted(ipc_info):
+                    f.write(b'%s: %s\n' % (key, ipc_info[key]))
 
     def release_lock(self):
-        if self.options.log_start:
-            log.info('FlexGet stopped (PID: %s)' % os.getpid())
         if os.path.exists(self.lockfile):
             os.remove(self.lockfile)
             log.debug('Removed %s' % self.lockfile)
         else:
             log.debug('Lockfile %s not found' % self.lockfile)
 
-    def create_tasks(self):
-        """Creates instances of all configured tasks"""
-        from flexget.task import Task
-        # Clear tasks dict
-        self.tasks = {}
-
-        # Backwards compatibility with feeds key
-        if 'feeds' in self.config:
-            log.warning('`feeds` key has been deprecated and replaced by `tasks`. Please update your config.')
-            if 'tasks' in self.config:
-                log.error('You have defined both `feeds` and `tasks`. Stop that.')
-            self.config['tasks'] = self.config.pop('feeds')
-        # construct task list
-        tasks = self.config.get('tasks', {}).keys()
-        for name in tasks:
-            # Make sure numeric task names are turned into strings. #1763, #1961
-            if not isinstance(name, basestring):
-                self.config['tasks'][unicode(name)] = self.config['tasks'].pop(name)
-                name = unicode(name)
-            # create task
-            task = Task(self, name, self.config['tasks'][name])
-            # if task name is prefixed with _ it's disabled
-            if name.startswith('_'):
-                task.enabled = False
-            self.tasks[name] = task
-
-    def disable_tasks(self):
-        """Disables all tasks."""
-        for task in self.tasks.itervalues():
-            task.enabled = False
-
-    def enable_tasks(self):
-        """Enables all tasks."""
-        for task in self.tasks.itervalues():
-            task.enabled = True
-
-    def process_start(self, tasks=None):
-        """Execute process_start for tasks.
-
-        :param list tasks: Optional list of :class:`~flexget.task.Task` instances, defaults to all.
-        """
-        if tasks is None:
-            tasks = self.tasks.values()
-
-        for task in tasks:
-            if not task.enabled:
-                continue
-            try:
-                log.trace('calling process_start on a task %s' % task.name)
-                task._process_start()
-            except Exception as e:
-                task.enabled = False
-                log.exception('Task %s process_start: %s' % (task.name, e))
-
-    def process_end(self, tasks=None):
-        """Execute process_end for all tasks.
-
-        :param list tasks: Optional list of :class:`~flexget.task.Task` instances, defaults to all.
-        """
-        if tasks is None:
-            tasks = self.tasks.values()
-
-        for task in tasks:
-            if not task.enabled:
-                continue
-            if task._abort:
-                continue
-            try:
-                log.trace('calling process_end on a task %s' % task.name)
-                task._process_end()
-            except Exception as e:
-                log.exception('Task %s process_end: %s' % (task.name, e))
-
-    @useExecLogging
-    def execute(self, tasks=None, disable_phases=None, entries=None):
-        """
-        Iterate trough tasks and run them. If --learn is used download and output
-        phases are disabled.
-
-        :param list tasks: Optional list of task names to run, all tasks otherwise.
-        :param list disable_phases: Optional list of phases to disabled
-        :param list entries: Optional list of entries to pass into task(s).
-            This will also cause task to disable input phase.
-        """
-        # Make a list of Task instances to execute
-        if tasks is None:
-            # Default to all tasks if none are specified
-            run_tasks = self.tasks.values()
-        else:
-            # Turn the list of task names or instances into a list of instances
-            run_tasks = []
-            for task in tasks:
-                if isinstance(task, basestring):
-                    if task in self.tasks:
-                        run_tasks.append(self.tasks[task])
-                    else:
-                        log.error('Task `%s` does not exist.' % task)
-                else:
-                    run_tasks.append(task)
-
-        if not run_tasks:
-            log.warning('There are no tasks to execute, please add some tasks')
+    def daemonize(self):
+        """Daemonizes the current process. Returns the new pid"""
+        if sys.platform.startswith('win'):
+            log.error('Cannot daemonize on windows')
             return
+        if threading.activeCount() != 1:
+            log.critical('There are %r active threads. '
+                         'Daemonizing now may cause strange failures.' % threading.enumerate())
 
-        disable_phases = disable_phases or []
-        # when learning, skip few phases
-        if self.options.learn:
-            log.info('Disabling download and output phases because of %s' %
-                     ('--reset' if self.options.reset else '--learn'))
-            disable_phases.extend(['download', 'output'])
+        log.info('Daemonizing...')
 
-        fire_event('manager.execute.started', self)
-        self.process_start(tasks=run_tasks)
+        try:
+            pid = os.fork()
+            if pid > 0:
+                # Don't run the exit handlers on the parent
+                atexit._exithandlers = []
+                # exit first parent
+                sys.exit(0)
+        except OSError as e:
+            sys.stderr.write('fork #1 failed: %d (%s)\n' % (e.errno, e.strerror))
+            sys.exit(1)
 
-        for task in sorted(run_tasks):
-            if not task.enabled or task._abort:
-                continue
-            try:
-                task.execute(disable_phases=disable_phases, entries=entries)
-            except Exception as e:
-                task.enabled = False
-                log.exception('Task %s: %s' % (task.name, e))
-            except KeyboardInterrupt:
-                # show real stack trace in debug mode
-                if self.options.debug:
-                    raise
-                print '**** Keyboard Interrupt ****'
-                return
+        # decouple from parent environment
+        os.chdir('/')
+        os.setsid()
+        os.umask(0)
 
-        self.process_end(tasks=run_tasks)
-        fire_event('manager.execute.completed', self)
+        # do second fork
+        try:
+            pid = os.fork()
+            if pid > 0:
+                # Don't run the exit handlers on the parent
+                atexit._exithandlers = []
+                # exit from second parent
+                sys.exit(0)
+        except OSError as e:
+            sys.stderr.write('fork #2 failed: %d (%s)\n' % (e.errno, e.strerror))
+            sys.exit(1)
 
-    def db_cleanup(self):
-        """ Perform database cleanup if cleanup interval has been met.
+        log.info('Daemonize complete. New PID: %s' % os.getpid())
+        # redirect standard file descriptors
+        sys.stdout.flush()
+        sys.stderr.flush()
+        si = file('/dev/null', 'r')
+        so = file('/dev/null', 'a+')
+        se = file('/dev/null', 'a+', 0)
+        os.dup2(si.fileno(), sys.stdin.fileno())
+        os.dup2(so.fileno(), sys.stdout.fileno())
+        os.dup2(se.fileno(), sys.stderr.fileno())
+
+    def db_cleanup(self, force=False):
         """
-        expired = self.persist.get('last_cleanup', datetime.now()) < datetime.now() - DB_CLEANUP_INTERVAL
-        if self.options.db_cleanup or not self.persist.get('last_cleanup') or expired and \
-                any([t.enabled for t in self.tasks.values()]):
-            if not self.options.db_cleanup and not self.options.quiet:
-                log.verbose('Not running database cleanup on manual run. It will be run on next --cron run.')
-                return
+        Perform database cleanup if cleanup interval has been met.
+
+        Fires events:
+
+        * manager.db_cleanup
+
+          If interval was met. Gives session to do the cleanup as a parameter.
+
+        :param bool force: Run the cleanup no matter whether the interval has been met.
+        """
+        expired = self.persist.get('last_cleanup', datetime(1900, 1, 1)) < datetime.now() - DB_CLEANUP_INTERVAL
+        if force or expired:
             log.info('Running database cleanup.')
             session = Session()
-            fire_event('manager.db_cleanup', session)
-            session.commit()
-            session.close()
+            try:
+                fire_event('manager.db_cleanup', session)
+                session.commit()
+            finally:
+                session.close()
             # Just in case some plugin was overzealous in its cleaning, mark the config changed
             self.config_changed()
             self.persist['last_cleanup'] = datetime.now()
         else:
             log.debug('Not running db cleanup, last run %s' % self.persist.get('last_cleanup'))
 
-    def shutdown(self):
-        """ Application is being exited
+    def shutdown(self, finish_queue=True):
         """
+        Application is being exited
+
+        :param bool finish_queue: Should scheduler finish the task queue
+        """
+        # Wait for scheduler to finish
+        self.scheduler.shutdown(finish_queue=finish_queue)
+        try:
+            self.scheduler.wait()
+        except KeyboardInterrupt:
+            log.debug('Not waiting for scheduler shutdown due to ctrl-c')
+            # show real stack trace in debug mode
+            if manager.options.debug:
+                raise
+            print '**** Keyboard Interrupt ****'
         fire_event('manager.shutdown', self)
         if not self.unit_test:  # don't scroll "nosetests" summary results when logging is enabled
             log.debug('Shutting down')
